unit ResizeblePanel;
{
  author: Wolfy 2001/02/02
  version: 0.5
  license: BSD
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,buttons;

{$R *.RES}

type
  THeaderType = (htDefault,htTop,htLeft);


  TResizeblePanel = class(TPanel)
  private
    { Private 錾 }
    FSplitter: TSplitter;
    FSplitterHeight: Integer;
    FSplitterWidth: Integer;
    FHeader: TPanel;
    FCloseButton: TSpeedButton;
    FBevel: TBevel;
    FHeaderType: THeaderType;

    function GetVisible: Boolean;
    procedure SetVisible(const Value: Boolean);
    function GetAlign: TAlign;
    procedure SetAlign(const Value: TAlign);
    function GetHeaderVisible: Boolean;
    procedure SetHeaderVisible(const Value: Boolean);
    function GetHeaderCaption: TCaption;
    procedure SetHeaderCaption(const Value: TCaption);

    procedure HeaderResize;
    //event
    procedure HeaderOnResize(Sender: TObject);
    procedure CloseOnClick(Sender: TObject);
    procedure SetHeaderType(const Value: THeaderType);
    
  protected
    { Protected 錾 }
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

  published
    { Published 錾 }
    property Visible : Boolean read GetVisible write SetVisible;
    property Align: TAlign read GetAlign write SetAlign;
    property Splitter: TSplitter read FSplitter;
    property SplitterHeight: Integer read FSplitterHeight write FSplitterHeight;
    property SplitterWidth: Integer read FSplitterWidth write FSplitterWidth;
    property CloseButton: TSpeedButton read FCloseButton;
    property HeaderVisible: Boolean read GetHeaderVisible write SetHeaderVisible;
    property HeaderCaption: TCaption read GetHeaderCaption write SetHeaderCaption;
    property HeaderType: THeaderType read FHeaderType write SetHeaderType;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Panel', [TResizeblePanel]);
end;

{ TResizeblePanel }

constructor TResizeblePanel.Create(AOwner: TComponent);
//쐬
var
  Bitmap: TBitmap;
begin
  inherited Create(AOwner);

  FSplitterHeight := 2;
  FSplitterWidth := 2;
  FHeaderType := htDefault;
  //header
  FHeader := TPanel.Create(Self);
  FHeader.OnResize := HeaderOnResize;
  FHeader.Parent := Self;
  FHeader.Align := alTop;
  FHeader.Top := 0;
  FHeader.Left := 0;
  FHeader.Height := 22;
  FHeader.Visible := True;
  FHeader.Enabled := True;
  FHeader.BorderStyle := bsNone;
  FHeader.BevelOuter := bvNone;
  FHeader.BevelInner := bvNone;
  FHeader.Alignment := taLeftJustify;
  SetHeaderVisible(True);
  //bevel
  FBevel := TBevel.Create(Self);
  FBevel.Parent := FHeader;
  FBevel.Align := alClient;
  FBevel.Style := bsLowered;
  FBevel.Shape := bsFrame;
  FBevel.Visible := True;
  FBevel.Enabled := True;
  //close button
  FCloseButton := TSpeedButton.Create(Self);
  FCloseButton.OnClick := CloseOnClick;
  // x}[N
  Bitmap := TBitmap.Create;
  try
    Bitmap.Handle := LoadBitmap(HInstance,'CLOSE');
    FCloseButton.Glyph.Assign(Bitmap);
  finally
    Bitmap.Free;
  end;

  FCloseButton.Parent := FHeader;
  FCloseButton.Height := 18;
  FCloseButton.Width := 18;
  FCloseButton.Flat := True;
  FCloseButton.Top := 2;
  FCloseButton.Enabled := True;
  FCloseButton.Visible := True;
  FCloseButton.Hint := '';
  FCloseButton.ShowHint := True;

  HeaderResize;

  FSplitter := TSplitter.Create(Self);
  FSplitter.Parent := Self.Parent;
  SetAlign(Align);
end;

destructor TResizeblePanel.Destroy;
//j
begin
  inherited;
end;

procedure TResizeblePanel.CloseOnClick(Sender: TObject);
begin
  Visible := False;
end;

function TResizeblePanel.GetAlign: TAlign;
begin
  Result := inherited Align;
end;

function TResizeblePanel.GetHeaderCaption: TCaption;
begin
  Result := Trim(FHeader.Caption);
end;

function TResizeblePanel.GetHeaderVisible: Boolean;
begin
  Result := FHeader.Visible;
end;

function TResizeblePanel.GetVisible: Boolean;
begin
  Result := inherited Visible;
end;

procedure TResizeblePanel.HeaderOnResize(Sender: TObject);
begin
  HeaderResize;
end;

procedure TResizeblePanel.HeaderResize;
const
  TopHeight = 22;
  LeftWidth = 22; //55
begin
  if Assigned(FCloseButton) then
  begin
    if FHeaderType = htTop then
    begin
      FHeader.Align := alTop;
      FHeader.Height := TopHeight;
      FCloseButton.Left := Self.Width - 21;
    end
    else if FHeaderType = htLeft then
    begin
      FHeader.Align := alLeft;
      FHeader.Width := LeftWidth;
      FCloseButton.Left := 2;
    end
    else begin
      if (Align = alLeft) or (Align = alRight) then
      begin
        FHeader.Align := alTop;
        FHeader.Height := TopHeight;
        FCloseButton.Left := Self.Width - 21;
      end
      else if (Align = alTop) or (Align = alBottom) then
      begin
        FHeader.Align := alLeft;
        FHeader.Width := LeftWidth;
        FCloseButton.Left := 2;
      end;
    end;

  end;
end;

procedure TResizeblePanel.SetAlign(const Value: TAlign);
begin
  inherited Align := Value;

  if Assigned(FSplitter) then
  begin
    FSplitter.Parent := Self.Parent;
    FSplitter.Align := Value;
    if Value = alTop then
    begin
      FSplitter.Height := FSplitterHeight;
      FSplitter.Top := Self.Top + Self.Height + 1;
      FSplitter.Left := Self.Left;
      FSplitter.Visible := True;
      FSplitter.Enabled := True;
    end
    else if Value = alLeft then
    begin
      FSplitter.Width := FSplitterWidth;
      FSplitter.Top := Self.Top;
      FSplitter.Left := Self.Left + Self.Width + 1;
      FSplitter.Visible := True;
      FSplitter.Enabled := True;
    end
    else if Value = alRight then
    begin
      FSplitter.Width := FSplitterWidth;
      FSplitter.Top := Self.Top;
      FSplitter.Left := Self.Left - 1;
      FSplitter.Visible := True;
      FSplitter.Enabled := True;
    end
    else if Value = alBottom then
    begin
      FSplitter.Height := FSplitterHeight;
      FSplitter.Top := Self.Top - 1;
      FSplitter.Left := Self.Left;
      FSplitter.Visible := True;
      FSplitter.Enabled := True;
    end
    else begin
      FSplitter.Visible := False;
      FSplitter.Enabled := False;
    end;
  end;

  HeaderResize;
end;

procedure TResizeblePanel.SetHeaderCaption(const Value: TCaption);
begin
  FHeader.Caption := '  ' + Value;
end;

procedure TResizeblePanel.SetHeaderVisible(const Value: Boolean);
begin
  FHeader.Visible := Value;
end;

procedure TResizeblePanel.SetVisible(const Value: Boolean);
//\
begin
  inherited Visible := Value;

  FSplitter.Visible := Value;
  if Value then
  begin
    SetAlign(Align);
    //HeaderResize;
  end;
end;


procedure TResizeblePanel.SetHeaderType(const Value: THeaderType);
begin
  if FHeaderType <> Value then
  begin
    FHeaderType := Value;
    //SetAlign(Align);
    HeaderResize;
  end;
end;

end.
